import enumerable from './decorators/enumerable';

export default class Store {
    static onError() { }

    @enumerable(false)
    debugger = false;

    static single = false; // 是否设置为单例

    /** 实例化对象应该使用init */
    static init(opts) {
        if (this.singleStore) {
            return this.singleStore;
        }
        const store = new this(opts);
        if (this.single) {
            this.singleStore = store;
        }
        return store;
    }

    constructor(opts = {}) {
        if (opts.props) this.props = opts.props;
        this.subscribers = [];

        this.initState();

        this.initActions();

        this.initEfects();

        this.initSelectors();
    }

    @enumerable(false)
    $beginChangeState = false;


    selectors = {};

    @enumerable(false)
    _state = {};

    @enumerable(false)
    _tempState = null;

    actions = {};

    register(fn) {
        this.subscribers.push(fn);
        return () => {
            this.subscribers = this.subscribers.filter(item => item !== fn);
        };
    }

    init() { } //eslint-disable-line

    willMount() { } //eslint-disable-line

    didMount() { } //eslint-disable-line

    willUnmount() { } //eslint-disable-line

    getState() {
        return this._state;
    }

    get state() {
        return this._state;
    }

    get name() {
        return this.constructor.name;
    }

    setProps(props) {
        this.props = props;
    }

    initState() {
        if (this.constructor.stateKeys) {
            this.constructor.stateKeys.forEach((key) => {
                this._state[key] = this[key];
            });
        }
    }

    initActions() {
        if (this.constructor.actionKeys) {
            this.constructor.actionKeys.forEach((key) => {
                this.actions[key] = this[key].bind(this);
            });
        }
    }

    /**
     * effects将自动生成有副作用的actions
     * 还有status
     *
     * @memberof ViewModel
     */
    initEfects() {
        this._effects = {};
        if (this.constructor.effectKeys) {
            this.constructor.effectKeys.forEach((key) => {
                this._effects[key] = this[key].bind(this);
                this._state.effects[key] = { pending: false, error: false };
            });
        } else {
            delete this._state.effects;
        }
    }

    initSelectors() {
        if (this.constructor.selectorKeys) {
            this.selectors = {};
            this.constructor.selectorKeys.forEach((key) => {
                const fn = this[key].bind(this);
                Object.defineProperty(this, key, {
                    get: () => fn(this.state, this.props),
                    enumerable: true,
                });
                Object.defineProperty(this.selectors, key, {
                    get: () => this[key],
                    enumerable: true,
                });
            });
        }
    }

    _updateState() {
        this._state = { ...this.state, ...this._tempState };
        this._tempState = null;
    }
}
