export default function state(store, key, descriptor) {
    if (!Object.prototype.hasOwnProperty.call(store.constructor, 'stateKeys')) {
        const stateKeys = Object.getPrototypeOf(store).constructor.stateKeys;
        store.constructor.stateKeys = stateKeys ? [...stateKeys] : [];
    }

    store.constructor.stateKeys.push(key);
    const initializer = descriptor.initializer;
    descriptor = {
        enumerable: true,
        get() {
            if (this._tempState && key in this._tempState) return this._tempState[key];
            if (key in this._state) return this._state[key];
            return initializer.call(this);
        },
        set(value) {
            if (this.$beginChangeState) {
                if (!this._tempState) this._tempState = {};
                this._tempState[key] = value;
            } else {
                throw new Error('You can only modify the state in the action function!');
            }
        },
    };
    return descriptor;
}
