import React from 'react';
import invariant from 'invariant';
import ReactStore from './react-store';

export default function shareStore(...Stores) {
    Stores.forEach((Store, index) => {
        if (!Array.isArray(Store)) Stores[index] = [Store, state => state];
    });
    Stores.forEach((Store) => {
        invariant(Store[0].single, 'When using shareStores, the store must be set to singleton');
        invariant((Store[0].init() instanceof ReactStore), 'You must use the Store');
    });
    return function createShareStoreContainer(C) {
        class ShareStoreContainer extends React.PureComponent {
            constructor(props) {
                super(props);
                this.stores = Stores.map(([Store]) => Store.init());
                this.unsubscribes = this.stores.map(store => store.register((state) => {
                    this.setState({
                        [store.name]: state,
                    });
                }));
            }

            componentWillUnmount() {
                this.unsubscribes.forEach(fn => fn());
            }

            render() {
                const props = this.stores.reduce((props, store, index) => Object.assign({}, props, Stores[index][1]({
                    ...store.state, ...store.selectors, ...store.actions, ...store._effects,
                })), this.props);
                return <C {...props} />;
            }
        }
        return ShareStoreContainer;
    };
}
