import { state, Store } from '../src';

describe('action', () => {
    class OrderStore extends Store {
        @state price = 3
        @state amount = 2
        @state orders = []
    }
    test('init state', () => {
        const orderStore = new OrderStore();
        expect(orderStore.state).toEqual({
            price: 3,
            amount: 2,
            orders: [],
        });
    });

    test('change state', () => {
        const orderStore = new OrderStore();
        orderStore._state = { price: 2, amount: 3, orders: [1, 2] };
        expect(orderStore.state).toEqual({ price: 2, amount: 3, orders: [1, 2] });
    });

    test('Can not modify state directly', () => {
        const orderStore = new OrderStore();
        expect(() => { orderStore.price = 1; }).toThrow();
    });

    test(' extend state ', () => {
        class SubTestStore extends OrderStore {
            @state
            sub = 1;
        }
        const subTestStore = new SubTestStore();
        expect(subTestStore.state).toEqual({
            price: 3,
            amount: 2,
            orders: [],
            sub: 1,
        });
    });
});
